/*
 * bufpointpainterarray.h
 * 
 * Buffered point painter array. Point cache for performant point drawing 
 * in different colors. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _INC_QTXLIB_BufferedPointPainterArray_H_
#define _INC_QTXLIB_BufferedPointPainterArray_H_ 1

#include "bufpointpainter.h"

namespace QTX
{

// This is a class which holds several BufferecdPointPainter classes. 
// This allows you e.g. to buffer points with different colors. 
// NOTE: You have to explicitly set up each BufferedPointPainter in 
//       the array after calling the constructor or Setup() before 
//       you can actually draw. 
class BufferedPointPainterArray
{
	protected:
		uint asize;
		BufferedPointPainter **pa;
	public:
		// See Setup() for explanation on the arg. 
		// See also NOTE at the beginning of the class. 
		BufferedPointPainterArray(uint array_size)
			{  pa=NULL; asize=0;  Setup(array_size);  }
		~BufferedPointPainterArray()
			{  Setup(0);  }
		
		uint GetArraySize() const
			{  return(asize);  }
		
		// Get one BufferedPointPainter explicitly (to set the color, etc). 
		// This returns NULL if pidx is out of range or the corresponding 
		// BufferedPointPainter is not set up. 
		BufferedPointPainter *BPP(uint pidx)
			{  return(pidx>=asize ? NULL : pa[pidx]);  }
		BufferedPointPainter *operator[](uint pidx)
			{  return(BPP(pidx));  }
		
		// This sets up the BufferedPointPainter array holding 
		// array_size elements. 
		// Set array_size=0 to clear. 
		void Setup(uint array_size);
		
		// Set up a BufferedPointPainter with index pidx in the array. 
		// The args after pidx are passed to the constructor of 
		// the BufferedPointPainter. 
		// If the index idx is already set up, the old BufferedPointPainter 
		// is flushed and destroyed and the new one put in place. 
		// Return value: 0 -> OK; 1 -> pidx out of range 
		int Setup(uint pidx,
			const QPaintDevice *pd,uint cache_size);
		int Setup(uint pidx,
			const QPainter *qp,uint cache_size);
		
		// Flush all BufferedPointPainter classes: 
		void Flush();
		
		// Flush specified BufferedPointPainter: 
		// Return value: 
		//    0 -> OK; 
		//    1 -> pidx out of range. 
		//    2 -> painter not set up
		int Flush(uint pidx);
		
		// Add a point to the correct BufferedPointPainter: 
		// pidx: painter index in range 0...array_size-1
		// Return value: 
		//    0 -> OK; 
		//    1 -> pidx out of range. 
		//    2 -> painter not set up
		int AddPoint(uint pidx,int x,int y)
		{
			if(pidx>=asize)  return(1);
			BufferedPointPainter *ppa=pa[pidx];  if(!ppa)  return(2);
			ppa->AddPoint(x,y);  return(0);
		}
};

}  // end of namespace QTX

#endif   /* _INC_QTXLIB_BufferedPointPainterArray_H_ */
