/*
 * rgbcolorsource.h
 * 
 * Converting RGB values to pixel values for visuals which do not need 
 * color allocation. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _INC_QTXLIB_RGBColorSource_H_
#define _INC_QTXLIB_RGBColorSource_H_ 1

#include "xpainter.h"

namespace QTX
{

/* Use a RGBColorSource to get the ulong color pixel values out 
 * of RGB spec of a color. 
 * You normally just need very few ColorSource instances. 
 * Note that only such visuals are supported which do NOT require color 
 * allocation, that means you need true color displays of the like 
 * (you should be safe with depths of 15, 16, 24, 32 bits). 
 */
class RGBColorSource
{
	private:
		ulong rmask,  gmask,   bmask;    // color mask
		ulong nred,   ngreen,  nblue;    // number of different colors
		ulong nred_1, ngreen_1,nblue_1;  // number of different colors - 1
		int   rshift, gshift,  bshift;   // bit shift value
		int   rss,    gss,     bss;      // rgb short int shift
		
		void _setup_cmask(ulong rm,ulong gm,ulong bm);
		void _setup(Visual *v);
		void _setup(const QPaintDevice *qpd)
			{  _setup((Visual*)qpd->x11Visual());  }
	public:
		RGBColorSource(const XPainter *data_from_here)
			{  _setup(data_from_here->qpaintdev());  }
		RGBColorSource(const QPaintDevice *data_from_here)
			{  _setup(data_from_here);  }
		RGBColorSource(const QPainter *data_from_here)
			{  _setup(data_from_here->device());  }
		// Take the red/green/blue mask from the XImgae: 
		RGBColorSource(XImage *img)
			{  _setup_cmask(img->red_mask,img->green_mask,img->blue_mask);  }
		~RGBColorSource() { }
		
		// Default copy constructor and operator=() may be used. 
		
		// Use these to translate the specified spec into a color value: 
		ulong color8(uint r,uint g,uint b)     // 0...255
		{  return(color16(r<<8,g<<8,b<<8));  }
		ulong color16(uint r,uint g,uint b)     // 0...65535
		{
			return(
				(ulong(r >> rss) << rshift) |
				(ulong(g >> gss) << gshift) |
				(ulong(b >> bss) << bshift) );
		}
		ulong color(float r,float g,float b)  // 0...1
		{
			return(
				((ulong(r * (nred_1))   << rshift) & rmask) |
				((ulong(g * (ngreen_1)) << gshift) & gmask) |
				((ulong(b * (nblue_1))  << bshift) & bmask) );
		}
		
};

}  // end of namespace QTX

#endif  /* _INC_QTXLIB_ColorSource_H_ */

