/*
 * burncore.h
 * 
 * BURN simulation core header file. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _BURN_BURNCORE_H_
#define _BURN_BURNCORE_H_ 1

#include <stdlib.h>

/* RANDON GENERATOR STUFF: */

#define UseLibcRandom 0

#if UseLibcRandom
#define RandomMax RAND_MAX
#else
#define RandomMax 2147483563
extern int random2(long *seed);
#endif

class RandomGenerator
{
	private:
		long seed;
		
	public:
		RandomGenerator();
		~RandomGenerator();
		
		// Returns a (pseudo)random number in range 0...RandomMax
		inline int get_random()
#if UseLibcRandom
			{  return(::random());  }
#else
			{  return(::random2(&seed));  }
#endif
		
		inline bool decide(int yes_prop)
			{  return(get_random()<yes_prop);  }
		
		// Convert a property in range 0...1 into an integer property: 
		static inline int PropD2I(double x)
		{  return(int(x*double(RandomMax)+0.5));  }
		
		// ...and vice versa: 
		static inline double PropI2D(int x)
		{  return(double(x)/double(RandomMax));  }
};



/* AREA CODE: */

// This is a burn cell (i.e. a `tree): 
struct BCell
{
	// The cell can have the following states: 
	enum State
	{
		Empty=0,
		Living,
		Burning,
		Dead,
		_LAST
	} state : 16;
	
	// This is the (remaining) `mass of the tree at this position. 
	// In state Burning, this value specifies how long the tree 
	// will still burn. 
	int mass : 16;
	
	// Pointer to next BCell with Burning state or NULL. 
	BCell *next;
}/*__attribute__((packed))*/;


// 1 -> Use state change array
// 0 -> use state change callback (Recommended: a lot faster)
#define USE_STATE_CHG 0


// This is the area consisting of burn cells. 
class BurnArea
{
	private:
		RandomGenerator rnd;
		
		BCell *cellarray;
		
		// List header for list containing all cells in state Burning
		BCell *first_brn;
		
		inline void _BRNAddFirst(BCell *c)
			{  c->next=first_brn;  first_brn=c;  }
		
		inline void _BRNDelete(BCell *c,BCell *prev)
		{
			if(prev)  prev->next=c->next;
			else  first_brn=first_brn->next;
		}
		
		// List header for list containing all cells which might start 
		// burning the next time: 
		BCell *first_flame;
		
		inline void _FLAMEAddFirst(BCell *c)
			{  c->next=first_flame;  first_flame=c;  }
		
		inline void _CheckCell(BCell *nc,unsigned char *bpf,unsigned char mask)
		{
			if(nc->state==BCell::Living)
			{  if(!(*bpf)) _FLAMEAddFirst(nc);
				(*bpf)|=mask;  }
		}
		
		// Counters: count how many cells are in specified state: 
		size_t ncell[BCell::_LAST];
		
		// The cell burn prop flag accumulator: 
		unsigned char *pbf_accu;
		
		// Size of the area: 
		int width,height;
		// Torus ares?
		int torus;
		
		// Burn propabilities: (for all neighbours [y][x])
		// Propability that cell in the center sets fire on the 
		// neighbour x,y: 
		double burn_prop[3][3];
		// Integer burn props for all 256 possible neighbour burn states: 
		int burn_prop_table[256];
		
		// Get array index of coordinates: 
		inline size_t arrayidx(int x,int y)  {  return(size_t(y)*width+x);  }
		// Get coordinates out of array index: 
		inline int idx2xco(size_t idx)  {  return(idx%width);  }
		inline int idx2yco(size_t idx)  {  return(idx/width);  }
		
		// Get cell at specified positon; no range check. 
		inline BCell *cell(int x,int y)
			{  return(cellarray+y*width+x);  }
		
		inline unsigned char *bpbfaccu(int x,int y)
			{  return(pbf_accu+y*width+x);  }
		
		#if USE_STATE_CHG
		// State change flags (one per cell): 
		unsigned char *state_chg;
		
		inline void _CellStateChanged(size_t idx)
			{   state_chg[idx]='\x1';  }
		//{
		//	ulong *ptr=(ulong*)state_chg;
		//	ptr[idx>>5]|=1LU<<(idx&31);
		//}
		#else
		void (*update_callback)(void *thisptr,int x,int y,BCell *c);
		void *callback_dptr;
		inline void _CellStateChanged(size_t idx)
			{  if(update_callback)  (*update_callback)(callback_dptr,
				idx2xco(idx),idx2yco(idx),cellarray+idx);  }
		#endif
		
		// See NOTE in area.cc for more details: 
		// ONLY CALL IF THE STATE REALLY _CHANGES_. 
		inline void _StateChangeUpdCnt(size_t idx,
			BCell::State old_state,BCell::State new_state)
		{
			--ncell[old_state];
			++ncell[new_state];
			_CellStateChanged(idx);
		}
		
		void _NeighboursAddBurnFlags_Center(int x,int y);
		void _NeighboursAddBurnFlags_Border(int x,int y);
		int _CellTick(BCell *sc,BCell *prev_sc);
		void _CellTickFire(BCell *sc);
		
		void _ResetCounters();
		void _CleanupArea();
	public:
		BurnArea();
		~BurnArea();
		
		// Query size: 
		int AreaWidth()  {  return(width);  }
		int AreaHeight()  {  return(height);  }
		int AreaIsTorus()  {  return(torus);  }
		
		// Query burn prop. *dest must be double [3][3]. 
		void GetBurnProp(double *dest);
		
		// Get number of cells in state: 
		// NONE OF THESE POINTERS MAY BE NULL. 
		void GetNCells(size_t *empty,size_t *living,size_t *burning,size_t *dead)
			{  *empty=ncell[BCell::Empty];  *living=ncell[BCell::Living];
			   *burning=ncell[BCell::Burning];  *dead=ncell[BCell::Dead];  }
		// Returns 1 if at least one cell is burning. 
		int HasBurningCells()
			{  return(ncell[BCell::Burning] ? 1 : 0);  }
		// Returns 1 if at least one cell is dead. 
		int HasDeadCells()
			{  return(ncell[BCell::Dead] ? 1 : 0);  }
		
		// Get a cell: [NO RANGE CHECK!]
		inline const BCell *AreaCell(int x,int y)
			{  return(cell(x,y));  }
		
		#if USE_STATE_CHG
		// Returns if state of cell was changed and resets the flag. 
		inline bool StateChangedReset(int x,int y)
		{
			unsigned char *flg=&state_chg[arrayidx(x,y)];
			if(*flg)
			{  *flg='\0';  return(true);  }
			return(false);
		}
		#else
		void RegisterUpdateCallback(
			void (*func)(void *thisptr,int x,int y,BCell *c),
			void *thisptr)
		{  update_callback=func;  callback_dptr=thisptr;  }
		#endif
		
		// No trees after a call to that function. 
		// Returns 0 on success. 
		int SetSize(int newwidth,int newheight,int torus_area);
		
		// Set the burn props (copied; *prop is an array with 9 
		// elements, the pointer to the first elem of an array 
		// prop[3][3] specifying the burn propability of neigbour 
		// [y][x]. 
		int SetBurnProp(const double *prop);
		
		// Fill the area. 
		// tree_prop: probability that the cell is a tree. 
		// mass: mass of the trees (must be >=1)
		void RandomFill(double tree_prop,int tree_mass);
		
		// Set fire on cell (x,y); 
		// Return value:
		//   0 -> state changed
		//   1 -> cell already burning
		//   2 -> x,y out of range
		//   3 -> otherwise... (cell state != Living or Burning)
		int SetFire(int x,int y);
		
		// Simulate one sim tick. 
		// Return value: 
		//  1 -> something changed
		//  0 -> nothing changed (stable state)
		int SimTick();
};

#endif  /* _BURN_BURNCORE_H_ */
