/*
 * burnprop.cc
 * 
 * Burn propability input dialog. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "burnprop.h"

#include <math.h>

#include <Qt/qlayout.h>
#include <Qt/qspinbox.h>
#include <Qt/qlabel.h>
#include <Qt/qtooltip.h>


void BurnPropDialog::prop_change()
{
	double bprop[3][3];
	
	// One of the burn props was changed. 
	for(int y=0; y<3; y++)
	{
		for(int x=0; x<3; x++)
		{
			if(x==1 && y==1)  continue;
			double v=double(burn_prop[y][x]->value())/100.0;
			if(v>1.0)  v=1.0;
			if(v<0.0)  v=0.0;
			bprop[y][x]=v;
		}
	}
	
	emit burn_prop_change(bprop[0]);
	
	_UpdateBurnProp();
}


void BurnPropDialog::windparam_change()
{
	double force=double(wp_force->value())/100.0 * 2.0;
	double dir=double(wp_direct->value())/180.0*M_PI;
	double combus=double(wp_combus->value())/100.0;
	
	// Compute new burn props: 
	// Direction like compass; 0 degrees -> North, then clockwise. 
	double xp=cos(-dir)*force;
	double yp=sin(-dir)*force;
	double f=sqrt(2.0);
	double bprop[3][3]=
	{
		{ (xp+yp)/f,  xp, (xp-yp)/f  },
		{    yp,     0.0,   -yp      },
		{ (yp-xp)/f, -xp, (-xp-yp)/f }
	};
	
	for(int y=0; y<3; y++)
	{
		for(int x=0; x<3; x++)
		{
			if(x==1 && y==1)  continue;
			double p=bprop[y][x];
			p=(1.0+p)*combus;
			if(p>1.0)  p=1.0;
			if(p<0.0)  p=0.0;
			bprop[y][x]=p;
		}
	}
	
	// Set burn props: 
	emit burn_prop_change(bprop[0]);
	
	// Write the new values back: 
	_UpdateBurnProp();
}


void BurnPropDialog::_UpdateBurnProp()
{
	double bprop[3][3];
	
	// Get burn props: 
	emit burn_prop_request(bprop[0]);
	
	for(int y=0; y<3; y++)
	{
		for(int x=0; x<3; x++)
		{
			if(x==1 && y==1)  continue;
			
			double v=bprop[y][x];
			burn_prop[y][x]->setValue(int(v*100.0+0.5));
		}
	}
}


BurnPropDialog::BurnPropDialog(QWidget *_parent,const char *name,WFlags f) : 
	QTabWidget(_parent,name,f)
{
	QWidget *wind_prop=new QWidget(this);
	QWidget *direct_prop=new QWidget(this);
	
	QWidget *parent=wind_prop;
	
	wp_force=new QSpinBox(0,100,5,parent);
	  wp_force->setSuffix("%");
	  connect(wp_force,SIGNAL(valueChanged(int)),this,SLOT(windparam_change()));
	wp_direct=new QSpinBox(0,359,10,parent);
	  wp_direct->setSuffix("");
	  connect(wp_direct,SIGNAL(valueChanged(int)),this,SLOT(windparam_change()));
	wp_combus=new QSpinBox(0,100,5,parent);
	  wp_combus->setSuffix("%");
	  wp_combus->setValue(50);
	  connect(wp_combus,SIGNAL(valueChanged(int)),this,SLOT(windparam_change()));
	
	int spacing=5;
	int margin=5;
	QWidget *wtmp;
	QGridLayout *wind_prop_layout=new QGridLayout(parent,3,2,margin,spacing);
	  wind_prop_layout->addWidget(wtmp=new QLabel(wp_force,"&force:",parent),0,0);
	  QToolTip::add(wtmp,"Wind force in percent");
	  wind_prop_layout->addWidget(wtmp=new QLabel(wp_direct,"dir&ection:",parent),1,0);
	  QToolTip::add(wtmp,"Wind direction (like compass\n"
	    "0 is North, then clockwise)");
	  wind_prop_layout->addWidget(wtmp=new QLabel(wp_combus,"&combustibility:",parent),2,0);
	  QToolTip::add(wtmp,"How good trees burn\n"
	    "(0%=bad .. 100%=good)");
	  wind_prop_layout->addWidget(wp_force,0,1);
	  wind_prop_layout->addWidget(wp_direct,1,1);
	  wind_prop_layout->addWidget(wp_combus,2,1);
	
	parent=direct_prop;
	QGridLayout *direct_prop_layout=new QGridLayout(parent,3,3,margin,spacing);
	direct_prop_layout->setColStretch(0,1);
	direct_prop_layout->setColStretch(1,1);
	direct_prop_layout->setColStretch(2,1);
	for(int y=0; y<3; y++)
		for(int x=0; x<3; x++)
		{
			if(x==1 && y==1)
			{
				burn_prop[y][x]=NULL;
				wtmp=new QLabel("[CELL]",parent);
				QHBoxLayout *clay=new QHBoxLayout();
				clay->addStretch(0);
				clay->addWidget(wtmp);
				clay->addStretch(0);
				direct_prop_layout->addLayout(clay,y,x);
				QToolTip::add(wtmp,
					"Burn propability per time tick\n"
					"for each neighbour cell.");
				continue;
			}
			
			burn_prop[y][x]=new QSpinBox(0,100,5,parent);
	  		burn_prop[y][x]->setSuffix("%");
			
			direct_prop_layout->addWidget(burn_prop[y][x],y,x);
			
			connect(burn_prop[y][x],SIGNAL(valueChanged(int)),
				this,SLOT(prop_change()));
		}
	
	this->addTab(wind_prop,"&Wind");
	this->addTab(direct_prop,"&Direct");
}

BurnPropDialog::~BurnPropDialog()
{
}
