/*
 * mainwindow.cc
 * 
 * Qxburn's main window. 
 * 
 * Copyright (c) 2002--2006 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "config.h"

#include "mainwindow.h"
#include "statwindow.h"

#include <Qt/qpushbutton.h>
#include <Qt/qcheckbox.h>
#include <Qt/qlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qlineedit.h>
#include <Qt/qvalidator.h>
#include <Qt/qspinbox.h>
#include <Qt/qgroupbox.h>
#include <Qt/qmessagebox.h>
#include <Qt/qtabwidget.h>
#include <Qt/qtooltip.h>
#include <Qt/qcombobox.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>


void MainWindow::quit_clicked()
{
	close();
}

void MainWindow::info_clicked()
{
	QMessageBox::about(this,
		"Burn Simulator",
		"<center><b>QX Burn</b>, version " VERSION "</center>"
		"Fast burn simulator<br>(c) 2002--2006 by Wolfgang Wieser.<br><br>"
		"This program may be copied unter the terms of the GNU GPL.<br>");
}


void MainWindow::clear_clicked()
{
	SimSetupPars ssp;  ssp.valid=false;
	atd->ReadValues(&ssp);
	assert(ssp.valid);
	
	areawin.ClearArea(&ssp);
	areawin.DoAutoIgnite(&ignitepars);
	if(!areawin.isVisible())
	{  areawin.Map();  }
	
	_UpdateValues();
	
	stop_cont_button->setText("&Start");
}


void MainWindow::delay_change()
{
	areawin.SetDelay(
		delay_mode->currentItem(),
		delay_time->value());
}


void MainWindow::stop_cont_clicked()
{
	if(areawin.Running())
	{
		areawin.StopSim();
		//stop_cont_button->setText("Cont");  // done by sim_has_stopped()
		//stop_cont_button->setAccel(ALT + Key_S);
	}
	else if(areawin.HasBurningTrees())
	{
		stop_cont_button->setText("&Stop");
		areawin.ContSim(n_iterations->value());
	}
}


void MainWindow::sim_has_stopped(int reason)
{
	// Called by sim window when the sim is stopped. 
	switch(reason)
	{
		case 1:  // no more trees 
			//stop_cont_button->setText("&Start");
			stop_cont_button->setText("Cont");
			stop_cont_button->setAccel(ALT + Key_S);
			break;
		case 0:  // StopSim() called
		case 2:  // iteration limit
			stop_cont_button->setText("Cont");
			stop_cont_button->setAccel(ALT + Key_S);
			break;
		default:  assert(0);
	}
}


void MainWindow::_UpdateValues()
{
	atd->Update();
	bpropd->Update();
}

void MainWindow::sim_setup_pars_request_slot(SimSetupPars *ssp)
{
	areawin.fillin_sim_setup(ssp);
}


void MainWindow::area_setup_changed(AreaSetupPars *spar)
{
	areawin.area_setup_changed(spar);
	clear_clicked();
}


void MainWindow::burn_prop_change(double *bprop)
{
	areawin.ChangeBurnProp(bprop);
}

void MainWindow::burn_prop_request_slot(double *bprop)
{
	areawin.fillin_burnprop(bprop);
}


// stat: +1 -> press; -1 -> release
void MainWindow::_HandleKeyEvent(QKeyEvent *ev,int stat)
{
	if(ev->isAutoRepeat() || stat==+1)
	{  ev->ignore();  return;  }
	
	switch(ev->key())
	{
		case Key_Return:
		case Key_Enter:
			// Let's see where we are: 
			if(atd->YouHaveFocus())
			{
				reset_button->animateClick();
				stop_cont_button->setFocus();
			}
			else if(!areawin.HasBurningTrees())
			{
				//reset_button->animateClick();
				atd->YouGetFocus(0);
			}
			else
			{
				stop_cont_button->animateClick();
			}
			break;
		case Key_A:  area_button->animateClick();  break;
		case Key_I:  ignite_button->animateClick();  break;
		case Key_R:  reset_button->animateClick();  break;
		case Key_S:  stop_cont_button->animateClick();  break;
		case Key_T:  atd->YouGetFocus(0);  break;
		case Key_M:  atd->YouGetFocus(1);  break;
		default:  ev->ignore();  return;
	}
}

void MainWindow::keyPressEvent(QKeyEvent *ev)
{
	_HandleKeyEvent(ev,+1);
}

void MainWindow::keyReleaseEvent(QKeyEvent *ev)
{
	_HandleKeyEvent(ev,-1);
}

// Called for key presses/releases from area window: 
void MainWindow::key_press_slot(QKeyEvent *ev)
{
	MainWindow::keyPressEvent(ev);
}

void MainWindow::key_release_slot(QKeyEvent *ev)
{
	MainWindow::keyReleaseEvent(ev);
}


void MainWindow::do_auto_ignite_req_now()
{
	// This gets called if auto ignition is enabled and the 
	// auto ignite dialog gets closed. 
	if(!areawin.HasBurningTrees() && !areawin.HasDeadTrees())
	{
		areawin.DoAutoIgnite(&ignitepars);
	}
}


void MainWindow::timerEvent(QTimerEvent *)
{
	/* only for setup */
	killTimers();
	
	_UpdateValues();
	delay_mode->setCurrentItem(0);
	delay_time->setValue(10);
	delay_change();
	// Make sure burn props are according to wind params: 
	bpropd->windparam_change();
	
	show();
	
	//areawin.Map();
	clear_clicked();
	
	stop_cont_button->setFocus();
}

MainWindow::MainWindow(QWidget *_parent,const char *name,WFlags _f) : 
	QWidget(_parent,name,_f),
	areawin(NULL),   // no parent
	asetup(NULL,NULL,WStyle_Customize | WStyle_DialogBorder),   // no parent
	ignitepars(NULL,NULL,WStyle_Customize | WStyle_DialogBorder)   // no parent
{
	QWidget *wtmp;
	
	connect(&asetup,SIGNAL(change_area_setup(AreaSetupPars*)),
		this,SLOT(area_setup_changed(AreaSetupPars*)));
	connect(&asetup,SIGNAL(get_area_setup(AreaSetupPars*)),
		&areawin,SLOT(fillin_area_setup(AreaSetupPars*)));
	
	connect(&ignitepars,SIGNAL(req_auto_ignite_now()),
		this,SLOT(do_auto_ignite_req_now()));
	
	connect(&areawin,SIGNAL(sim_has_stopped(int)),
		this,SLOT(sim_has_stopped(int)));
	connect(&areawin,SIGNAL(key_press_event(QKeyEvent *)),
		this,SLOT(key_press_slot(QKeyEvent *)));
	connect(&areawin,SIGNAL(key_release_event(QKeyEvent *)),
		this,SLOT(key_release_slot(QKeyEvent *)));
	
	/******** HEAD ********/
	QWidget *parent=this;
	QPushButton *quit_button=new QPushButton("&Quit",parent);
	connect(quit_button,SIGNAL(clicked()),this,SLOT(quit_clicked()));
	
	QPushButton *info_button=new QPushButton("Info",parent);
	connect(info_button,SIGNAL(clicked()),this,SLOT(info_clicked()));
	
	QHBoxLayout *head_layout=new QHBoxLayout();
	  head_layout->addWidget(quit_button);
	  head_layout->addSpacing(2);
	  head_layout->addWidget(info_button);
	
	
	int margin=5;
	int spacing=5;
	/******** SIM CONTROL ********/
	QGroupBox *sim_ctrl_group=new QGroupBox("Sim Control",this);
	parent=sim_ctrl_group;
	
	n_iterations=new QSpinBox(0,999999,1,parent);
	  n_iterations->setValue(0);
	  n_iterations->setSpecialValueText("[no limit]");
	
	QString _ctrl_explanation(
		"Control sim speed:\n"
		"msec delay/tick -> pause specified number of\n"
		"           milli-seconds after each sim tick\n"
		"ticks per delay -> do 10 millisecond delay after\n"
		"           specified numbers of sim ticks [use this\n"
		"           if 1 msec delay/tick is too slow]");
	delay_time=new QSpinBox(0,10000,10,parent);
	  delay_time->setSpecialValueText("[none]");
	  connect(delay_time,SIGNAL(valueChanged(int)),this,SLOT(delay_change()));
	  QToolTip::add(delay_time,_ctrl_explanation);
	delay_mode=new QComboBox(parent);
	  delay_mode->insertItem("msec delay/tick",0);
	  delay_mode->insertItem("ticks per delay",1);
	  connect(delay_mode,SIGNAL(activated(int)),this,SLOT(delay_change()));
	  QToolTip::add(delay_mode,_ctrl_explanation);
	
	reset_button=new QPushButton("&Reset",parent);
	  connect(reset_button,SIGNAL(clicked()),
		this,SLOT(clear_clicked()));
      QToolTip::add(reset_button,
        "Fill area with green trees according to\n"
	    "tree density; do auto-ignition if enabled.");
	stop_cont_button=new QPushButton("&Start",parent);
	  stop_cont_button->setAccel(ALT + Key_S);
	  connect(stop_cont_button,SIGNAL(clicked()),
		this,SLOT(stop_cont_clicked()));
      QToolTip::add(stop_cont_button,"Start / stop / continue sim.");
	
	QString _titer(
		"Halt sim after this number of time ticks.\n"
		"Special value 0 means 'unlimited'.");
	QGridLayout *simctl_inplayout=new QGridLayout(2,2);
	  simctl_inplayout->addWidget(wtmp=new QLabel(n_iterations,
		"<- time tick limit",parent),0,1);
	  QToolTip::add(wtmp,_titer);
	  simctl_inplayout->addWidget(n_iterations,0,0);
	  simctl_inplayout->addWidget(delay_mode,1,1);
	  simctl_inplayout->addWidget(delay_time,1,0);
	
	QHBoxLayout *sctl_bt_layout=new QHBoxLayout();
	  sctl_bt_layout->addWidget(reset_button);
	  sctl_bt_layout->addWidget(stop_cont_button);
	
	QVBoxLayout *sctl_layout=new QVBoxLayout(parent,margin,spacing);
	  sctl_layout->addSpacing(15);
	  sctl_layout->addLayout(simctl_inplayout);
	  sctl_layout->addLayout(sctl_bt_layout);
	
	/******** PARAMETERS ********/
	QGroupBox *area_group=new QGroupBox("Parameters",this);
	parent=area_group;
	
	atd=new AreaTreeDialog(1,parent);
	  connect(atd,SIGNAL(sim_setup_pars_request(SimSetupPars *)),
	    this,SLOT(sim_setup_pars_request_slot(SimSetupPars *)));
	
	ignite_button=new QPushButton("&Ignition...",parent);
	  connect(ignite_button,SIGNAL(clicked()),&ignitepars,SLOT(map()));
	  QToolTip::add(ignite_button,"Parameters for automatic ignition.");
	
	area_button=new QPushButton("&Area...",parent);
	  connect(area_button,SIGNAL(clicked()),&asetup,SLOT(map()));
	  QToolTip::add(area_button,"Specify area width and height.");
	
	QHBoxLayout *area_bt_layout=new QHBoxLayout();
	  area_bt_layout->addWidget(ignite_button);
	  area_bt_layout->addWidget(area_button);
	
	QVBoxLayout *area_inplayout=new QVBoxLayout(parent,margin,spacing);
	  area_inplayout->addSpacing(15);
	  area_inplayout->addWidget(atd);
	  area_inplayout->addLayout(area_bt_layout);
	
	
	/******** STATUS ********/
	
	swin=new StatusWindow(this);
	areawin.SetStatWindow(swin);
	
	/******** PROPS ********/
	
	bpropd=new BurnPropDialog(this);
	  connect(bpropd,SIGNAL(burn_prop_change(double*)),
		this,SLOT(burn_prop_change(double*)));
	  connect(bpropd,SIGNAL(burn_prop_request(double*)),
		this,SLOT(burn_prop_request_slot(double*)));
	
	/******** TOP LAYOUT ********/
	QVBoxLayout *toplayout=new QVBoxLayout(this,/*margin=*/3,/*spacing=*/0);
	  toplayout->addLayout(head_layout);
	  toplayout->addSpacing(2);
	  toplayout->addWidget(sim_ctrl_group);
	  toplayout->addWidget(area_group);
	  toplayout->addWidget(swin);
	  toplayout->addSpacing(5);
	  toplayout->addWidget(bpropd);
	
	startTimer(0);
}


MainWindow::~MainWindow()
{
	delete swin;
}
