/*
 * statdiag.cc
 * 
 * Statistics diagram. 
 * 
 * Copyright (c) 2002--2003 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "statdiag.h"

#include <stdio.h>
#include <math.h>
#include <assert.h>


inline double SQR(double x)
	{  return(x*x);  }

void LinesCache::EnsureSize(int min_size)
{
	assert((size%2)==0);  // A line always has a start and an end.
	if(size<min_size)
	{
		size=min_size;
		if(pnts)  free(pnts);
		pnts=(XPoint*)malloc(size*sizeof(XPoint));
		assert(pnts);
	}
	Reset();
}

void LinesCache::DoDraw(QTX::XPainter *pnt)
{
	int npoints=head-pnts;
	if(npoints>1)
	{  pnt->DrawLines(pnts,head-pnts,CoordModeOrigin);  }
	else if(npoints==1)
	{
		XPoint *n=pnts+1;
		pnt->DrawLine(pnts->x,pnts->y,n->x,n->y);
	}
	Reset();
}

inline void LinesCache::AddLine(int x0,int y0,int x1,int y1)
{
	assert(pnts);
	assert(head+1<pnts+size);
	
	head->x=x0;
	head->y=y0;
	++head;
	head->x=x1;
	head->y=y1;
	++head;
}


// Draw dashed diagram lines: 
// dir: 'h' -> horizontal; 'v' -> vertical
// ndiv -> number of divisions
void StatisticsDiagram::_DrawDiagramLines(int dir,int ndiv)
{
	// This draws horiz. lines; swaps coos for vert lines. 
	int diag_height = dir=='h' ? this->diag_height : this->diag_width;
	int diag_width = dir=='h' ? this->diag_width : this->diag_height;
	int diag_x0 = dir=='h' ? this->diag_x0 : this->diag_y0;
	int diag_y0 = dir=='h' ? this->diag_y0 : this->diag_x0;
	
	char tmp[16];
	for(int i=0; i<=ndiv; i++)
	{
		int y=int(double(diag_height)*i/ndiv + 0.5);
		if(dir=='h')  y=diag_height-y;
		y+=diag_y0;
		
		if(dir=='h')
		{
			framepnt.DrawLine(diag_x0           -1 -2,y,diag_x0           -1 +2,y);
			framepnt.DrawLine(diag_x0+diag_width+1 -2,y,diag_x0+diag_width+1 +2,y);
		}  else  {
			framepnt.DrawLine(y,diag_x0           -1 -2,y,diag_x0           -1 +2);
			framepnt.DrawLine(y,diag_x0+diag_width+1 -2,y,diag_x0+diag_width+1 +2);
		}
		
		snprintf(tmp,16,"%.2f",double(i)/double(ndiv));
		if(dir=='h')
		{  framepnt.DrawString(diag_x0-1 -26,y+4,tmp);  }
		else
		{  framepnt.DrawString(y-10,diag_x0+diag_width+12,tmp);  }
		
		if(i!=0 && i!=ndiv)
		{
			int tmpx=diag_x0-1 +2;
			int xend=diag_x0+diag_width+1 -2  -4;
			for(;;)
			{
				tmpx+=20;
				if(tmpx>=xend)  break;
				if(dir=='h')
				{  framepnt.DrawLine(tmpx,y,tmpx+4,y);  }
				else
				{  framepnt.DrawLine(y,tmpx,y,tmpx+4);  }
			}
		}
	}
	
}


void StatisticsDiagram::_RedrawAll()
{
	if(!ismapped) return;
	
	// Clear the window: 
	backpnt.ClearWindow();
	
	// Draw the frame: 
	framepnt.DrawRectangle(diag_x0-1,diag_y0-1,diag_width+2,diag_height+2);
	framepnt.DrawString(diag_x0-28,diag_y0-8,"Fraction of burnt trees, ");
	framepnt.DrawString(diag_x0+diag_width-57,diag_y0+diag_height+23,
		"Tree density");
	if(do_draw_avg_curve>1 && B_div_ent)
	{  varpnt.DrawString(diag_x0+121,diag_y0-8,"Variance");  }
	if(do_draw_duration_curve && B_div_ent)
	{  durationpnt.DrawString(diag_x0+121+56,diag_y0-8,"Duration");  }
	
	{
		int x0=diag_x0+diag_width/2-50,y0=diag_y0+diag_height+23;
		framepnt.DrawString(x0,y0,"Key:");
		ppnt->DrawString(x0+30,y0,"e");
		curvepnt.DrawString(x0+40,y0,"c");
		varpnt.DrawString(x0+50,y0,"c");
		durationpnt.DrawString(x0+60,y0,"d");
	}
	
	_DrawNEntText(1,1,1);
	
	_DrawDiagramLines('h',10);
	_DrawDiagramLines('v',10);
	
	// Draw the dots: 
	if(do_draw_dots)
	{
		for(EntryNode *n=first; n; n=n->next)
		{
			for(SEntry *se=n->ent,*seend=se+n->nent; se<seend; se++)
			{
				ppnt->AddPoint(
					Prop2DiagX(se->tree_prop),Prop2DiagY(se->dead_frac));
			}
		}
		
		ppnt->Flush();
	}
	
	// Draw the curve: 
	if(do_draw_avg_curve && B_div_ent)
	{
		// It's easy to use XDrawLines() here becuase I know the number 
		// of lines quite well...
		avg_lines.EnsureSize(4*B_n_div_entries);   // -> 2 lines per entry
		
		// We do not draw anything for an entry if we have nent=0 for an 
		// entry so that a gap in the curve remains. 
		for(DivEntry *de=B_div_ent,*deend=de+B_n_div_entries; de<deend; de++)
		{
			if(finite(de->sum_dead_frac) && de->nent)
			{
				int yy=Prop2DiagY(de->sum_dead_frac/double(de->nent));
				int x0=Prop2DiagX(de->min_tree_prop);
				int x1=Prop2DiagX(de->max_tree_prop);
				avg_lines.AddLine(x0,yy,x1,yy);
				
				// Draw variance if needed: 
				if(do_draw_avg_curve>1 && de->nent>=2)  // need at least 2 entries
				{
					double sigma = sqrt( (de->sq_sum_dead_frac - 
						SQR(de->sum_dead_frac)/double(de->nent)) / 
						double(de->nent-1) );
					int sigma_pix=int(sigma/2*diag_height +0.5);
					if(sigma_pix>0)
					{
						XPoint pntbuf[6];
						XPoint *pnt=pntbuf;
						int xm=(x0+x1)/2;
						int ymin=yy-sigma_pix;
						int ymax=yy+sigma_pix;
						pnt->x=xm-1;  pnt->y=ymin;  ++pnt;
						pnt->x=xm+1;  pnt->y=ymin;  ++pnt;
						pnt->x=xm;  pnt->y=ymin;  ++pnt;
						pnt->x=xm;  pnt->y=ymax;  ++pnt;
						pnt->x=xm-1;  pnt->y=ymax;  ++pnt;
						pnt->x=xm+1;  pnt->y=ymax;  ++pnt;
						
						varpnt.DrawLines(pntbuf,6,CoordModeOrigin);
					}
				}
			}
			else
			{  avg_lines.DoDraw(&curvepnt);  }
		}
		
		// Draw the lines using the passed painter: 
		avg_lines.DoDraw(&curvepnt);
	}
	
	// Draw the duration curve: 
	if(do_draw_duration_curve && B_div_ent)
	{
		// It's easy to use XDrawLines() here becuase I know the number 
		// of lines quite well...
		dur_lines.EnsureSize(2*B_n_div_entries);   // -> 1 line per entry
		
		// First, find max. avg duration: 
		double max_avg_dur=1.0;  // This is sane...
		for(DivEntry *de=B_div_ent,*deend=de+B_n_div_entries; de<deend; de++)
		{
			if(!de->nent)  continue;
			double avg_dur=double(de->sum_duration)/double(de->nent);
			if(max_avg_dur<avg_dur)
			{  max_avg_dur=avg_dur;  }
		}
		
		// We do not draw anything for an entry if we have nent=0 for an 
		// entry so that a gap in the curve remains. 
		for(DivEntry *de=B_div_ent,*deend=de+B_n_div_entries; de<deend; de++)
		{
			if(de->nent)
			{
				int x0=Prop2DiagX(de->min_tree_prop);
				int x1=Prop2DiagX(de->max_tree_prop);
				double avg_dur=double(de->sum_duration)/double(de->nent);
				int yy=Prop2DiagY(avg_dur/max_avg_dur);
				dur_lines.AddLine(x0,yy,x1,yy);
			}
			else
			{  dur_lines.DoDraw(&durationpnt);  }
		}
		
		// Draw the lines using the passed painter: 
		dur_lines.DoDraw(&durationpnt);
	}
	
	gettimeofday(&last_curve_update,NULL);   // YES!!! NOT in the if(...) block above!
}


void StatisticsDiagram::_DrawNEntText(bool all,bool do_nent,bool do_sps)
{
	// Position of number, NOT of "n=". 
	int ty0=diag_y0-8;
	int tx0_n=diag_x0+diag_width-130;
	int tx0_s=diag_x0+diag_width-40;
	
	if(all)
	{
		framepnt.DrawString(tx0_n-14,ty0,"n=");
		framepnt.DrawString(tx0_s-26,ty0,"sps=");  // simulations per second
	}
	
	char tmp[32];
	
	if(do_nent)
	{
		backpnt.FillRectangle(tx0_n,ty0-10,10*6+2,12);
		snprintf(tmp,32,"%u",nent);
		ppnt->DrawString(tx0_n,ty0,tmp);
	}
	
	if(do_sps)
	{
		backpnt.FillRectangle(tx0_s,ty0-10,10*8,12);
		snprintf(tmp,32,"%.1f",sims_per_sec);
		ppnt->DrawString(tx0_s,ty0,tmp);
	}
}


// Copied from HLib (c) Wolfgang Wieser...
static inline long msec_elapsed(const struct timeval *old,const struct timeval *current)
{
	return(
		(current->tv_sec  - old->tv_sec )*1000L + 
		(current->tv_usec - old->tv_usec)/1000L );
}

void StatisticsDiagram::_DrawAddEntry(StatisticsDiagram::SEntry *se,
	StatisticsDiagram::SEntry *undraw_se,bool redraw_sps)
{
	if(!(change_ent%10))
	{
		timeval current;
		gettimeofday(&current,NULL);
		if(msec_elapsed(&last_curve_update,&current)>=500)
		{
			_RedrawAll();
			return;
		}
	}
	
	if(do_draw_dots)
	{
		if(undraw_se)
		{
			backpnt.DrawPoint(
				Prop2DiagX(undraw_se->tree_prop),
				Prop2DiagY(undraw_se->dead_frac));
		}
		
		ppnt->DrawPoint(
			Prop2DiagX(se->tree_prop),
			Prop2DiagY(se->dead_frac));
	}
	
	bool draw_nent=!(change_ent%25);
	if(draw_nent || redraw_sps)
	{  _DrawNEntText(0,draw_nent,redraw_sps);  }
}


void StatisticsDiagram::_AddDivArrayEntry(int n_div_entries,DivEntry *div_ent,
	SEntry *se)
{
	if(finite(se->dead_frac))
	{
		int idx=int(se->tree_prop*n_div_entries);
		if(idx<0)  idx=0;
		else if(idx>=n_div_entries)  idx=n_div_entries-1;
		DivEntry *de=&div_ent[idx];
		++de->nent;
		de->sum_dead_frac+=se->dead_frac;
		de->sq_sum_dead_frac+=se->dead_frac*se->dead_frac;
		de->sum_duration+=double(se->duration);
	}
}

void StatisticsDiagram::_DelDivArrayEntry(int n_div_entries,DivEntry *div_ent,
	SEntry *se)
{
	if(finite(se->dead_frac))
	{
		int idx=int(se->tree_prop*n_div_entries);
		if(idx<0)  idx=0;
		else if(idx>=n_div_entries)  idx=n_div_entries-1;
		DivEntry *de=&div_ent[idx];
		--de->nent;
		de->sum_dead_frac-=se->dead_frac;
		de->sq_sum_dead_frac-=se->dead_frac*se->dead_frac;
		de->sum_duration-=double(se->duration);
	}
}


inline StatisticsDiagram::SEntry *StatisticsDiagram::_AllocNextEntry()
{
	// Allocate new entry node if needed: 
	if(!last || last->nent>=ENTRIES_PER_NODE)
	{
		EntryNode *n=(EntryNode*)malloc(sizeof(EntryNode));
		assert(n);
		
		n->nent=0;
		n->next=NULL;
		
		if(!last)
		{  first=last=n;  }
		else
		{  last->next=n;  last=n;  }
	}
	
	++nent;
	return(&(last->ent[last->nent++]));
}


inline void StatisticsDiagram::_UpdateMinMaxPtrs(StatisticsDiagram::SEntry *se)
{
	if(!duration_max)
	{
		duration_max=se;
		dead_frac_max=se;
		dead_frac_min=se;
	}
	else
	{
		if(dead_frac_max->dead_frac<se->dead_frac)
		{  dead_frac_max=se;  }
		else if(dead_frac_min->dead_frac>se->dead_frac)
		{  dead_frac_min=se;  }

		if(duration_max->duration<se->duration)
		{  duration_max=se;  }
	}
}


void StatisticsDiagram::AddRecord(double tree_prop,BurnArea *area,
	int iterations,long usec)
{
	size_t empty,living,burning,dead;
	area->GetNCells(&empty,&living,&burning,&dead);
	
	// If the area does not contain trees, we don't record it: 
	if(!(living+dead))  return;
	
	SEntry *se;
	SEntry undraw_se;  // only for entry limit
	int need_undraw=0;
	
	if(entry_limit && nent>=entry_limit)
	{
		assert(nent==entry_limit);
		
		if(!el_write_pos)
		{
			el_write_node=first;
			el_write_pos=el_write_node->ent;
		}
		
		// Overwrite this entry. Also remove it from the div array 
		// and the diagram [latter done below]. 
		se=el_write_pos;
		
		// Update write position: 
		++el_write_pos;
		if(el_write_pos>=el_write_node->ent+el_write_node->nent)
		{
			el_write_node=el_write_node->next;
			if(!el_write_node)  el_write_node=first;
			el_write_pos=el_write_node->ent;
		}
		
		_DelDivArrayEntry(A_n_div_entries,A_div_ent,se);
		_DelDivArrayEntry(B_n_div_entries,B_div_ent,se);
		
		undraw_se=*se;   // implicit copy
		need_undraw=1;
	}
	else
	{  se=_AllocNextEntry();  }
	
	//assert(!burning);
	se->tree_prop=tree_prop;
	se->dead_frac=double(dead)/double(living+dead);
	se->duration=iterations;
	
	if(!entry_limit)
	{
		// NOTE: these cannot be efficiently maintained if 
		// entry_limit is switched on and thus remain NULL 
		// in this case. 
		_UpdateMinMaxPtrs(se);
	}
	
	_AddDivArrayEntry(A_n_div_entries,A_div_ent,se);
	_AddDivArrayEntry(B_n_div_entries,B_div_ent,se);
	
	++change_ent;
	int redraw_sps=0;
	if(usec>=0)
	{
		#if 0
		// I get usec= 285 278 278 197 250 243 1 324 400 331 1 221 2 1 
		// 273 285 1 2 1 91 343 183 1 191 1 1 1 210 231 370 23 231 366 
		// 352 1 196 1 1 335 221 10 252 357 252 1 1 387 373 242 2 314 
		// 242 295 1 372 386 246 279 296 365 126 182 6 357 286 297 2 
		// 199 360 2 275 384 220 3 315 392 347 2 1 136 301 404 5 1 2 
		// 1 146 302 2 1 2 345 245 1 11 358 202 371 1 228 25 1 390 
		// 362 1 1 1 1 1 10 208 1 405 213 332 389 317 1 1 23 217 
		// ... so this method does not work for that (numbers too high). 
		fprintf(stderr,"%ld ",usec/1000);
		// cfact: how fast sims_per_sec reacts on changes. 
		// smaller values -> slower
		double cfact = 0.2;
		//if(!finite(cfact) || cfact>0.1 || cfact<0.0)  cfact=0.1;
		sims_per_sec = (1.0-cfact)*sims_per_sec + 1000000.0*cfact/double(usec);
		#else
		sps_accu+=double(usec)/1000000.0;
		++sps_accu_nent;
		if(sps_accu_nent>=10 && sps_accu>=1.0)
		{
			sims_per_sec = double(sps_accu_nent)/sps_accu;
			sps_accu=double(usec)/1000000.0;
			sps_accu_nent=1;
			redraw_sps=1;
		}
		#endif
	}
	
	_DrawAddEntry(se,need_undraw ? &undraw_se : NULL,redraw_sps);
}


inline void StatisticsDiagram::_RelimitAddRecord(StatisticsDiagram::SEntry *data)
{
	// Special function used by ChangeEntryLimit()
	assert(entry_limit && nent<entry_limit);
	
	SEntry *se=_AllocNextEntry();
	*se=*data;  // implicit copy
	
	_AddDivArrayEntry(A_n_div_entries,A_div_ent,se);
	_AddDivArrayEntry(B_n_div_entries,B_div_ent,se);
	// NO DrawAddEntry()... [does _RedrawAll() after everything]
}


void StatisticsDiagram::_ResetDivArray(int n_div_entries,DivEntry *div_ent)
{
	for(int i=0; i<n_div_entries; i++)
	{
		DivEntry *de=&div_ent[i];
		de->min_tree_prop=double(i)/double(n_div_entries);
		de->max_tree_prop=double(i+1)/double(n_div_entries);
		de->nent=0;
		de->sum_dead_frac=0.0;
		de->sq_sum_dead_frac=0.0;
		de->sum_duration=0.0;
	}
}

void StatisticsDiagram::Reset()
{
	last=NULL;
	while(first)
	{
		EntryNode *n=first;
		first=first->next;
		free(n);
	}
	nent=0;
	change_ent=0;
	
	sps_accu=0.0;
	sps_accu_nent=0;
	
	el_write_node=NULL;
	el_write_pos=NULL;
	
	duration_max=NULL;
	dead_frac_max=NULL;
	dead_frac_min=NULL;
	
	_ResetDivArray(A_n_div_entries,A_div_ent);
	_ResetDivArray(B_n_div_entries,B_div_ent);
	
	ppnt->Clear();  // be sure...
	
	_RedrawAll();
}


void StatisticsDiagram::ChangeEntryLimit(size_t new_limit)
{
	if(new_limit==entry_limit)  return;
	
	// Switching off limit is easy: 
	if(entry_limit && !new_limit)
	{
		entry_limit=0;
		el_write_node=NULL;
		el_write_pos=NULL;
		
		// Re-calc min/max: 
		for(EntryNode *n=first; n; n=n->next)
		{
			for(SEntry *se=n->ent,*seend=se+n->nent; se<seend; se++)
			{  _UpdateMinMaxPtrs(se);  }
		}
		
		return;
	}
	
	// If switching on limit: 
	if(!entry_limit)
	{
		// These are not maintained when limiting entries. 
		duration_max=NULL;
		dead_frac_max=NULL;
		dead_frac_min=NULL;
	}
	
	// HERE: switching on limit or resizing limit. 
	assert(new_limit!=0);
	if((new_limit>entry_limit && nent<entry_limit) || 
	   ((new_limit<entry_limit || !entry_limit) && nent<=new_limit) )
	{
		// Easy case as limit was not yet reached. 
		assert(!el_write_pos);
		entry_limit=new_limit;
		return;
	}
	
	// This is done by resetting and re-adding the data. 
	EntryNode *fnode=first;
	size_t old_nent=nent;
	size_t old_entry_limit=entry_limit;
	EntryNode *old_el_write_node=el_write_node;
	SEntry *old_el_write_pos=el_write_pos;
	first=NULL;  // so that reset will...
	last=NULL;   // ... not free it
	
	Reset();
	entry_limit=new_limit;
	
	if(!old_entry_limit || !old_el_write_node)
	{
		// Switch on limit: 
		assert(old_nent>=new_limit);
		size_t may_skip = old_nent - new_limit;
		for(EntryNode *n=fnode; n; n=n->next)
		{
			if(may_skip>=size_t(n->nent))
			{  may_skip-=size_t(n->nent);  continue;  }
			for(SEntry *se=n->ent+may_skip,*seend=n->ent+n->nent; se<seend; se++)
			{  _RelimitAddRecord(se);  }
			may_skip=0;
		}
	}
	else if(new_limit<old_entry_limit)
	{
		// Make limit smaller: 
		EntryNode *orig_el_write_node=old_el_write_node;
		SEntry *orig_el_write_pos=old_el_write_pos;
		
		assert(old_nent>=new_limit);
		size_t may_skip = old_nent - new_limit;
		for(;;)
		{
			size_t avail = (old_el_write_node->ent + old_el_write_node->nent) - 
				old_el_write_pos;
			if(avail>may_skip)
			{
				old_el_write_pos+=may_skip;
				may_skip=0;
				break;
			}
			
			may_skip-=avail;
			old_el_write_node=old_el_write_node->next;
			if(!old_el_write_node)  old_el_write_node=fnode;
			old_el_write_pos=old_el_write_node->ent;
		}
		
		// This loop is a bit inefficient but... oh, well.
		for(size_t i=0; i<new_limit; i++)
		{
			_RelimitAddRecord(old_el_write_pos);
			++old_el_write_pos;
			if(old_el_write_pos>=old_el_write_node->ent+old_el_write_node->nent)
			{
				old_el_write_node=old_el_write_node->next;
				if(!old_el_write_node)  old_el_write_node=fnode;
				old_el_write_pos=old_el_write_node->ent;
			}
		}
		
		assert(orig_el_write_node==old_el_write_node);
		assert(orig_el_write_pos==old_el_write_pos);
	}
	else
	{
		// Make limit larger: 
		assert(new_limit>old_entry_limit);
		assert(old_nent<=new_limit);
		
		// Re-add all entries (in correct order): 
		EntryNode *n=old_el_write_node;
		SEntry *se=old_el_write_pos;
		for(SEntry *seend=n->ent+n->nent; se<seend; se++)
		{  _RelimitAddRecord(se);  }
		
		for(;;)
		{
			n=n->next;
			if(!n)  n=fnode;
			se=n->ent;
			if(n==old_el_write_node)  break;
			for(SEntry *seend=se+n->nent; se<seend; se++)
			{  _RelimitAddRecord(se);  }
		}
		
		for(; se<old_el_write_pos; se++)
		{  _RelimitAddRecord(se);  }
	}
	
	// Free nodes: 
	while(fnode)
	{
		EntryNode *n=fnode;
		fnode=fnode->next;
		free(n);
	}
	
	_RedrawAll();
}


void StatisticsDiagram::paintEvent(QPaintEvent *ev)
{
	QWidget::paintEvent(ev);
	
	_RedrawAll();
}


void StatisticsDiagram::_HandleKeyEvent(QKeyEvent *ev,int stat)
{
	if(ev->isAutoRepeat() || stat==+1)
	{  ev->ignore();  return;  }
	
	switch(ev->key())
	{
		case Key_E:  // entries
			do_draw_dots=!do_draw_dots;
			_RedrawAll();
			break;
		case Key_C:  // curve
			do_draw_avg_curve=(do_draw_avg_curve+1)%3;
			_RedrawAll();
			break;
		case Key_D:  // duration
			do_draw_duration_curve=!do_draw_duration_curve;
			_RedrawAll();
			break;
		default:  ev->ignore();  return;
	}
}

void StatisticsDiagram::keyPressEvent(QKeyEvent *ev)
{
	_HandleKeyEvent(ev,+1);
}

void StatisticsDiagram::keyReleaseEvent(QKeyEvent *ev)
{
	_HandleKeyEvent(ev,-1);
}


void StatisticsDiagram::resizeEvent(QResizeEvent *)
{
	_RecalcSize(size().width(),size().height());
	_RedrawAll();
}

void StatisticsDiagram::_RecalcSize(int nw,int nh)
{
	diag_x0=40;
	diag_y0=20;
	diag_width=nw-60;
	diag_height=nh-48;
}


void StatisticsDiagram::area_setup_changed(AreaSetupPars * /*spar*/)
{
	// Changed. So what?
}


StatisticsDiagram::StatisticsDiagram(QWidget *parent,const char *name,WFlags f) : 
	QWidget(parent,name,f),
	QTX::XPainter(this),
	backpnt(this),
	framepnt(this),
	curvepnt(this),
	varpnt(this),
	durationpnt(this),
	avg_lines(),
	dur_lines()
{
	ismapped=0;
	
	fnt=XLoadFont(dpy,"fixed");
	framepnt.SetFont(fnt);
	
	setBackgroundColor(Qt::white);
	backpnt.SetForeground(paletteBackgroundColor().pixel());
	
	ppnt=new QTX::BufferedPointPainter(this,4096);
	
	QPainter pnt(this);
	pnt.setBrush(QBrush(Qt::blue,Qt::SolidPattern));
	ppnt->QCopyGC(pnt,1);
	ppnt->SetFont(fnt);
	
	pnt.setBrush(QBrush(Qt::black,Qt::SolidPattern));
	framepnt.QCopyGC(pnt,1);
	
	pnt.setBrush(QBrush(Qt::red,Qt::SolidPattern));
	curvepnt.QCopyGC(pnt,1);
	
	pnt.setBrush(QBrush(QColor(0xff,0xa7,0x00),Qt::SolidPattern));
	varpnt.QCopyGC(pnt,1);
	
	pnt.setBrush(QBrush(QColor(0x00,0x82,0x00),Qt::SolidPattern));
	durationpnt.QCopyGC(pnt,1);
	
	A_n_div_entries=10;
	A_div_ent=(DivEntry *)malloc(A_n_div_entries*sizeof(DivEntry));
	B_n_div_entries=100;
	B_div_ent=(DivEntry *)malloc(B_n_div_entries*sizeof(DivEntry));
	
	gettimeofday(&last_curve_update,NULL);
	
	first=NULL;
	last=NULL;
	entry_limit=2000;
	
	sims_per_sec=0.0;
	
	do_draw_dots=1;
	do_draw_avg_curve=1;
	do_draw_duration_curve=1;
	
	Reset();
	
	int minw=320+120,minh=120;
	setMinimumSize(QSize(minw,minh));
	
	_RecalcSize(width(),height());
}


StatisticsDiagram::~StatisticsDiagram()
{
	XUnloadFont(dpy,fnt);
	delete ppnt;
	Reset();
	if(A_div_ent)  free(A_div_ent);
	if(B_div_ent)  free(B_div_ent);
}
